/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.model.fixings;

import org.apache.log4j.Logger;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.DataFacet;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;

/**
 * Facet to show average W values for Q sectors.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixLongitudinalReferenceFacet extends DataFacet implements FacetTypes {

    private static final long serialVersionUID = 1L;

    /** House log. */
    private static Logger log = Logger.getLogger(FixLongitudinalReferenceFacet.class);

    private int columnId;

    /** Trivial Constructor. */
    public FixLongitudinalReferenceFacet() {
    }

    public FixLongitudinalReferenceFacet(final int facetIndex, final int columnId, final String name, final String description) {
        super(facetIndex, name, description, ComputeType.ADVANCE, null, null);

        this.columnId = columnId;
    }

    /**
     * Returns the data this facet requires.
     *
     * @param artifact
     *            the owner artifact.
     * @param context
     *            the CallContext.
     *
     * @return the data as KMIndex.
     */
    @Override
    public Object getData(final Artifact artifact, final CallContext context) {
        log.debug("FixLongitudinalReferenceFacet.getData");

        if (artifact instanceof D4EArtifact) {
            final D4EArtifact flys = (D4EArtifact) artifact;

            final CalculationResult res = (CalculationResult) flys.compute(context, ComputeType.ADVANCE, false);

            final FixAnalysisResult result = (FixAnalysisResult) res.getData();

            final FixResultColumns resultColumns = result.getFixResultColumns();
            final FixResultColumn event = resultColumns.getColumn(this.columnId);
            if (event == null) {
                log.error("Missing event witrh columnId: " + this.columnId);
                return null;
            }

            return event.getQWDs();
        }

        log.warn("Artifact is no instance of D4EArtifact.");
        return null;
    }

    /**
     * Create a deep copy of this Facet.
     *
     * @return a deep copy.
     */
    @Override
    public FixLongitudinalReferenceFacet deepCopy() {
        final FixLongitudinalReferenceFacet copy = new FixLongitudinalReferenceFacet();
        copy.set(this);
        copy.columnId = this.columnId;
        return copy;
    }
}